<?php

namespace Drupal\forum_notifications_subscription\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\forum_notifications_subscription\ForumNotificationsSubscriptionServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller responses subscribe and unsubscribe action links.
 *
 * The response is a set of AJAX commands to update the
 * link in the page.
 */
class ActionLinkController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The forum notifications subscription service.
   *
   * @var \Drupal\forum_notifications_subscription\ForumNotificationsSubscriptionServiceInterface
   */
  protected $forumNotificationsService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructs a new ActionLinkController object.
   *
   * @param \Drupal\forum_notifications_subscription\ForumNotificationsSubscriptionServiceInterface $forum_notifications_service
   *   The forum notifications subscription service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(ForumNotificationsSubscriptionServiceInterface $forum_notifications_service, EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory, AccountInterface $current_user) {
    $this->forumNotificationsService = $forum_notifications_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('forum_notifications_subscription.frequency'),
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  /**
   * Subscribe to the entity.
   *
   * @param string|null $entity_type_id
   *   The entity type ID.
   * @param string|null $entity_id
   *   The entity ID.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The AJAX response object.
   */
  public function subscribe(?string $entity_type_id = NULL, ?string $entity_id = NULL) {
    $this->forumNotificationsService->createNotificationFrequencyByEntity($entity_id, $entity_type_id);
    return $this->getResponse($entity_type_id, $entity_id);
  }

  /**
   * Unsubscribe from the entity.
   *
   * @param string|null $entity_type_id
   *   The entity type ID.
   * @param string|null $entity_id
   *   The entity ID.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The AJAX response object.
   */
  public function unsubscribe(?string $entity_type_id = NULL, ?string $entity_id = NULL) {
    $this->forumNotificationsService->deleteNotificationFrequencyByEntityAndType($entity_id, $entity_type_id);
    return $this->getResponse($entity_type_id, $entity_id);
  }

  /**
   * Returns the subscription link.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity for which the subscription link is generated.
   * @param string $type
   *   The type of the entity (e.g., 'Forum topic' or 'Forum').
   * @param string $link_selector
   *   The selector for the link element.
   * @param \Drupal\forum_notifications_subscription\ForumNotificationsSubscriptionServiceInterface|null $forum_notifications_service
   *   The forum notifications subscription service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface|null $config_factory
   *   The config factory.
   * @param \Drupal\Core\Session\AccountInterface|null $current_user
   *   The current user.
   *
   * @return array
   *   A render array representing the subscription link.
   */
  public static function getSubscriptionLink(ContentEntityInterface $entity, string $type, string $link_selector, ?ForumNotificationsSubscriptionServiceInterface $forum_notifications_service = NULL, ?ConfigFactoryInterface $config_factory = NULL, ?AccountInterface $current_user = NULL): array {
    // Fallback to legacy service calls for backward compatibility.
    if (!$forum_notifications_service) {
      $forum_notifications_service = \Drupal::service('forum_notifications_subscription.frequency');
    }
    if (!$config_factory) {
      $config_factory = \Drupal::service('config.factory');
    }
    if (!$current_user) {
      $current_user = \Drupal::service('current_user');
    }

    $entity_type_id = $entity->getEntityTypeId();
    $label_setting_key = $entity_type_id == 'node' ? 'topic' : 'forum';
    $config = $config_factory->get('forum_notifications_subscription.settings');

    if ($forum_notifications_service->currentUserHasNotificationFrequencyByEntity($entity, $type)) {
      $route_name = 'forum_notifications_subscription.action_link_unsubscription';
      $title = $config->get($label_setting_key . '_label_off');
    }
    else {
      $route_name = 'forum_notifications_subscription.action_link_subscription';
      $title = $config->get($label_setting_key . '_label_on');
    }

    $link = [
      '#type' => 'link',
      '#title' => $title,
      '#url' => Url::fromRoute($route_name, [
        'entity_type_id' => $entity_type_id,
        'entity_id' => $entity->id(),
      ]),
      '#attributes' => [
        'id' => $link_selector,
        'class' => ['use-ajax', 'btn', 'btn-primary'],
      ],
      '#attached' => [
        'library' => ['core/drupal.ajax'],
      ],
    ];

    $cacheable_metadata = new CacheableMetadata();
    $cacheable_metadata->setCacheContexts(['route']);
    $cacheable_metadata->addCacheableDependency($entity);
    $cacheable_metadata->addCacheableDependency($config);
    $cacheable_metadata->addCacheableDependency($current_user);
    $cacheable_metadata->applyTo($link);
    return $link;
  }

  /**
   * Returns the response for the AJAX request.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $entity_id
   *   The entity ID.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The AJAX response object.
   */
  protected function getResponse(string $entity_type_id, string $entity_id) {
    $entity = $this->entityTypeManager->getStorage($entity_type_id)->load($entity_id);
    if ($entity_type_id == 'node') {
      $wrapper_class = 'forum-topic-subscription';
      $type = 'Forum topic';
    }
    else {
      $wrapper_class = 'forum-main-subscription';
      $type = 'Forum';
    }

    $link_selector = $wrapper_class . '-' . $entity->id();
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand("#{$link_selector}", static::getSubscriptionLink(
      $entity,
      $type,
      $link_selector,
      $this->forumNotificationsService,
      $this->configFactory,
      $this->currentUser
    )));
    return $response;
  }

}
