<?php

namespace Drupal\forum_notifications_subscription\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for forum notifications subscription settings.
 *
 * This form provides administrative interface for configuring:
 * - Forum subscription button labels and email templates
 * - Forum topic subscription settings and notifications
 * - Daily digest email configurations
 * - Global notification preferences including cron settings.
 */
class ForumNotificationsSubscriptionSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'forum_notifications_subscription_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['forum_notifications_subscription.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('forum_notifications_subscription.settings');
    $dropdown_array = [
      'Single Emails' => $this->t('Single Emails'),
      'Daily Digest Emails' => $this->t('Daily Digest Emails'),
    ];

    // @todo remove the settings entry in the config and use update hook to remove old config data.
    $form['forum_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Forum options settings'),
    ];
    $form['forum_settings']['forum_label_on'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enter the label of the forum subscription button.'),
      '#default_value' => $config->get('forum_label_on') ?? $config->get('settings.forum_label_on'),
      '#required' => TRUE,
    ];
    $form['forum_settings']['forum_label_off'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enter the label of the forum unsubscription button.'),
      '#default_value' => $config->get('forum_label_off') ?? $config->get('settings.forum_label_off'),
      '#required' => TRUE,
    ];
    $form['forum_settings']['email_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Email settings'),
    ];
    // Add the token tree UI.
    $form['forum_settings']['email_settings']['forum_token_help'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => ['fns_topic'],
      '#global_types' => FALSE,
    ];
    $form['forum_settings']['email_settings']['post_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subject'),
      '#description' => $this->t('The email subject.'),
      '#default_value' => $config->get('post_subject') ?? $config->get('settings.post_subject'),
      '#required' => TRUE,
    ];
    $form['forum_settings']['email_settings']['post_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Single Message'),
      '#description' => $this->t('The single email body.'),
      '#default_value' => $config->get('post_message') ?? $config->get('settings.post_message'),
      '#required' => TRUE,
      '#resizeable' => TRUE,
    ];
    $form['forum_settings']['email_settings']['post_dd_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Daily Digest Message'),
      '#description' => $this->t('The daily digest email body.'),
      '#default_value' => $config->get('post_dd_message') ?? $config->get('settings.post_dd_message'),
      '#required' => TRUE,
      '#resizeable' => TRUE,
    ];
    $form['forum_settings']['email_settings']['forum_default_frequency'] = [
      '#type' => 'select',
      '#title' => $this->t('Forum email default frequency'),
      '#description' => $this->t('The default frequency for forum email notification. * Single Emails (To send directly the notification when a forum topic is added), * Daily Digest Emails (This option will send emails at the middle night of each user time zone)'),
      '#default_value' => $config->get('forum_default_frequency') ?? $config->get('settings.forum_default_frequency'),
      '#options' => $dropdown_array,
      '#required' => TRUE,
    ];

    $form['topic_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Forum topic options settings'),
    ];
    $form['topic_settings']['topic_label_on'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enter the label of the topic subscription button.'),
      '#default_value' => $config->get('topic_label_on') ?? $config->get('settings.topic_label_on'),
      '#required' => TRUE,
    ];
    $form['topic_settings']['topic_label_off'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enter the label of the topic unsubscription button.'),
      '#default_value' => $config->get('topic_label_off') ?? $config->get('settings.topic_label_off'),
      '#required' => TRUE,
    ];
    $form['topic_settings']['email_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Email settings'),
    ];
    // Add the token tree UI.
    $form['topic_settings']['email_settings']['topic_token_help'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => ['fns_comment'],
      '#global_types' => FALSE,
    ];
    $form['topic_settings']['email_settings']['comment_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subject'),
      '#description' => $this->t('The email subject.'),
      '#default_value' => $config->get('comment_subject') ?? $config->get('settings.comment_subject'),
      '#required' => TRUE,
    ];
    $form['topic_settings']['email_settings']['comment_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Single Message'),
      '#description' => $this->t('The Single email body.'),
      '#default_value' => $config->get('comment_message') ?? $config->get('settings.comment_message'),
      '#required' => TRUE,
      '#resizeable' => TRUE,
    ];
    $form['topic_settings']['email_settings']['comment_dd_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Daily Digest Message'),
      '#description' => $this->t('The daily digest email body.'),
      '#default_value' => $config->get('comment_dd_message') ?? $config->get('settings.comment_dd_message'),
      '#required' => TRUE,
      '#resizeable' => TRUE,
    ];
    $form['topic_settings']['email_settings']['topic_default_frequency'] = [
      '#type' => 'select',
      '#title' => $this->t('Forum topic email default frequency'),
      '#description' => $this->t('The default frequency for forum topic email notification. * Single Emails (To send directly the notification when a forum topic is commented), * Daily Digest Emails (This option will send emails at the middle night of each user time zone)'),
      '#default_value' => $config->get('topic_default_frequency') ?? $config->get('settings.topic_default_frequency'),
      '#options' => $dropdown_array,
      '#required' => TRUE,
    ];
    $form['global'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Global settings'),
    ];
    // Add the token tree UI.
    $form['global']['dd_token_help'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => ['fns_dd'],
      '#global_types' => FALSE,
    ];
    $form['global']['dd_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Daily Digest Subject'),
      '#description' => $this->t('The daily digest email subject.'),
      '#default_value' => $config->get('dd_subject') ?? $config->get('settings.dd_subject'),
      '#required' => TRUE,
    ];
    $form['global']['dd_header_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Daily Digest Header Message'),
      '#description' => $this->t('The daily digest header body.'),
      '#default_value' => $config->get('dd_header_message') ?? $config->get('settings.dd_header_message'),
      '#required' => TRUE,
      '#resizeable' => TRUE,
    ];
    $form['global']['cron'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Send single emails when cron run'),
      '#description' => $this->t('Send single emails when cron run, this is useful when you want to avoid slow page saving.'),
      '#default_value' => $config->get('cron') ?? $config->get('settings.cron'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Currently no custom validation is implemented.
    // Add validation logic here as needed for form fields.
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('forum_notifications_subscription.settings');

    // Forum settings.
    $config->set('forum_label_on', $form_state->getValue('forum_label_on'));
    $config->set('forum_label_off', $form_state->getValue('forum_label_off'));
    $config->set('post_subject', $form_state->getValue('post_subject'));
    $config->set('post_message', $form_state->getValue('post_message'));
    $config->set('post_dd_message', $form_state->getValue('post_dd_message'));
    $config->set('forum_default_frequency', $form_state->getValue('forum_default_frequency'));

    // Topic settings.
    $config->set('topic_label_on', $form_state->getValue('topic_label_on'));
    $config->set('topic_label_off', $form_state->getValue('topic_label_off'));
    $config->set('comment_subject', $form_state->getValue('comment_subject'));
    $config->set('comment_message', $form_state->getValue('comment_message'));
    $config->set('comment_dd_message', $form_state->getValue('comment_dd_message'));
    $config->set('topic_default_frequency', $form_state->getValue('topic_default_frequency'));

    // Global settings.
    $config->set('dd_subject', $form_state->getValue('dd_subject'));
    $config->set('dd_header_message', $form_state->getValue('dd_header_message'));
    $config->set('cron', $form_state->getValue('cron'));

    $config->save();

    parent::submitForm($form, $form_state);
  }

}
