<?php

namespace Drupal\forum_notifications_subscription;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Service for managing forum notification subscriptions.
 *
 * This service handles creating, reading, updating, and deleting
 * forum notification frequency entities for users subscribed to
 * forum topics and forums.
 */
class ForumNotificationsSubscriptionService implements ForumNotificationsSubscriptionServiceInterface {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  private $currentUser;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * The forum notifications subscription settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private $settings;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Constructs a new ForumNotificationsSubscriptionService object.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user account.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(AccountInterface $current_user, EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory, LoggerInterface $logger) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->settings = $config_factory->get('forum_notifications_subscription.settings');
    $this->logger = $logger;
  }

  /**
   * Checks notification frequency for a given entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to check notification frequency for.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The notification frequency entity if found, NULL otherwise.
   */
  public function checkNotificationFrequencyByEntity($entity) {
    try {
      $frequency = $this->entityTypeManager->getStorage('forum_notification_frequency')
        ->loadByProperties([
          'entity_id' => $entity->id(),
          'user_id' => $this->currentUser->id(),
        ]);
      $frequency = reset($frequency);
    }
    catch (\Exception $e) {
      $this->logger->alert('Cant get entities with id: ' . $entity->id() . ' it do not exist');
      return NULL;
    }

    return $frequency;
  }

  /**
   * Checks if user has notification frequency for given entity and type.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to check.
   * @param string $type
   *   The notification type ('Forum topic' or 'Forum').
   *
   * @return bool
   *   TRUE if the user has a notification frequency set, FALSE otherwise.
   */
  public function currentUserHasNotificationFrequencyByEntity(ContentEntityInterface $entity, string $type): bool {
    $storage = $this->entityTypeManager->getStorage('forum_notification_frequency');
    return (bool) $storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', $type)
      ->condition('entity_id', $entity->id())
      ->condition('user_id', $this->currentUser->id())
      ->count()
      ->execute();
  }

  /**
   * Creates a notification frequency entity for the current user.
   *
   * @param int $entity_id
   *   The ID of the entity to create notification for.
   * @param string $entity_type_id
   *   The entity type ID ('node' or 'taxonomy_term').
   */
  public function createNotificationFrequencyByEntity($entity_id, $entity_type_id) {
    if ($entity_type_id == 'node') {
      $entity = $this->entityTypeManager->getStorage($entity_type_id)
        ->load($entity_id);
      $type = 'Forum topic';
      $default_frequency = $this->settings->get('topic_default_frequency') ?? $this->settings->get('settings.topic_default_frequency');
    }
    elseif ($entity_type_id == 'taxonomy_term') {
      $entity = $this->entityTypeManager->getStorage($entity_type_id)
        ->load($entity_id);
      $type = 'Forum';
      $default_frequency = $this->settings->get('forum_default_frequency') ?? $this->settings->get('settings.forum_default_frequency');
    }
    $frequency = $this->entityTypeManager->getStorage('forum_notification_frequency')
      ->create([
        'name' => $this->currentUser->getAccountName(),
        'user_id' => $this->currentUser->id(),
        'entity_id' => $entity->id(),
        'entity_name' => $entity->label(),
        'type' => $type,
        'frequency' => $default_frequency,
      ]);

    $frequency->save();
  }

  /**
   * Creates a notification frequency entity for a specific user.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to create notification for.
   * @param \Drupal\Core\Session\AccountInterface $user
   *   The user account to create the notification for.
   */
  public function createNotificationFrequencyForUser($entity, $user) {
    if ($entity->getEntityTypeId() == 'node') {
      $type = 'Forum topic';
      $default_frequency = $this->settings->get('forum_default_frequency') ?? $this->settings->get('settings.forum_default_frequency');
    }
    elseif ($entity->getEntityTypeId() == 'taxonomy_term') {
      $type = 'Forum';
      $default_frequency = $this->settings->get('forum_default_frequency') ?? $this->settings->get('settings.forum_default_frequency');
    }
    $frequency = $this->entityTypeManager->getStorage('forum_notification_frequency')
      ->create([
        'name' => $user->getAccountName(),
        'user_id' => $user->id(),
        'entity_id' => $entity->id(),
        'entity_name' => $entity->label(),
        'type' => $type,
        'frequency' => $default_frequency,
      ]);

    $frequency->save();
  }

  /**
   * Deletes notification frequency for current user by entity ID.
   *
   * @param int $entity_id
   *   The ID of the entity to delete notification frequency for.
   */
  public function deleteNotificationFrequencyByEntity($entity_id) {
    try {
      $frequency = $this->entityTypeManager->getStorage('forum_notification_frequency')
        ->loadByProperties([
          'entity_id' => $entity_id,
          'user_id' => $this->currentUser->id(),
        ]);
      $frequency = reset($frequency);

      $frequency->delete();
    }
    catch (\Exception $e) {
      $this->logger->alert('Cant delete entity with id: ' . $entity_id . ' it do not exist');
    }
  }

  /**
   * Deletes all notification frequencies for a given entity.
   *
   * This removes notification subscriptions for all users associated
   * with the specified entity.
   *
   * @param int $entity_id
   *   The entity ID.
   *
   * @throws \Exception
   *   If the entity does not exist.
   */
  public function deleteNotificationFrequencyByEntityAll($entity_id) {
    try {
      $frequencies = $this->entityTypeManager
        ->getStorage('forum_notification_frequency')
        ->loadByProperties(['entity_id' => $entity_id]);
      foreach ($frequencies as $id => $frequency) {
        $frequency->delete();
      }
    }
    catch (\Exception $e) {
      $this->logger->alert('Cant delete frequencies for entity id: ' . $entity_id . ' it do not exist');
    }
  }

  /**
   * Deletes all notification frequencies for a given user.
   *
   * This removes all forum notification subscriptions for the specified user.
   *
   * @param int $user_id
   *   The user ID.
   */
  public function deleteNotificationFrequenciesByUser($user_id) {
    try {
      $frequencies = $this->entityTypeManager
        ->getStorage('forum_notification_frequency')
        ->loadByProperties(['user_id' => $user_id]);
      foreach ($frequencies as $id => $frequency) {
        $frequency->delete();
      }
    }
    catch (\Exception $e) {
      $this->logger->alert('Cant delete frequencies for user id: ' . $user_id . ' it do not exist');
    }
  }

  /**
   * Deletes notification frequency by entity ID and type for current user.
   *
   * @param int $entity_id
   *   The ID of the entity.
   * @param string $entity_type_id
   *   The entity type ID ('node' or 'taxonomy_term').
   */
  public function deleteNotificationFrequencyByEntityAndType($entity_id, $entity_type_id) {
    $type = $entity_type_id == 'node' ? 'Forum topic' : 'Forum';
    try {
      $properties = [
        'entity_id' => $entity_id,
        'user_id' => $this->currentUser->id(),
        'type' => $type,
      ];
      $frequency = $this->entityTypeManager->getStorage('forum_notification_frequency')
        ->loadByProperties($properties);
      $frequency = reset($frequency);

      $frequency->delete();
    }
    catch (\Exception $e) {
      $this->logger->alert('Cant delete entity with id: ' . $entity_id . ' it do not exist');
    }
  }

  /**
   * Gets notification frequencies by entity ID and type.
   *
   * @param int $entity_id
   *   The ID of the entity.
   * @param string $entity_type_id
   *   The entity type ID ('node' or 'taxonomy_term').
   *
   * @return \Drupal\Core\Entity\EntityInterface[]|null
   *   An array of notification frequency entities, or NULL if none found.
   */
  public function getNotificationFrequencyByEntityAndType($entity_id, $entity_type_id) {
    $type = $entity_type_id == 'node' ? 'Forum topic' : 'Forum';
    try {
      $frequencies = $this->entityTypeManager->getStorage('forum_notification_frequency')
        ->loadByProperties([
          'entity_id' => $entity_id,
          'type' => $type,
        ]);
    }
    catch (\Exception $e) {
      $this->logger->alert('Cant get entities with id: ' . $entity_id . ' it do not exist');
      return NULL;
    }

    return $frequencies;
  }

  /**
   * Gets notification entities for a user by type.
   *
   * @param string $type
   *   The notification type ('Forum topic' or 'Forum').
   * @param int $user_id
   *   The user ID. If 0, uses current user.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]|null
   *   An array of notification frequency entities, or NULL if none found.
   */
  public function getUserNotificationEntities($type, $user_id = 0) {
    try {
      $frequencies = $this->entityTypeManager->getStorage('forum_notification_frequency')
        ->loadByProperties([
          'user_id' => $user_id ?: $this->currentUser->id(),
          'type' => $type,
        ]);
    }
    catch (\Exception $e) {
      $this->logger->alert('Cant get entities for user with id: ' . $user_id ?: $this->currentUser->id());
      return NULL;
    }

    return $frequencies;
  }

  /**
   * Gets a notification frequency entity by its ID.
   *
   * @param int $id
   *   The notification frequency entity ID.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The notification frequency entity if found, NULL otherwise.
   */
  public function getNotificationById($id) {
    try {
      $frequency = $this->entityTypeManager->getStorage('forum_notification_frequency')
        ->loadByProperties([
          'id' => $id,
        ]);
      $frequency = reset($frequency);
    }
    catch (\Exception $e) {
      $this->logger->alert('Cant get entity with id: ' . $id . ' it do not exist');
      return NULL;
    }

    return $frequency;
  }

}
