<?php

namespace Drupal\forum_notifications_subscription\Plugin\QueueWorker;

use Drupal\Component\Datetime\DateTimePlus;
use Drupal\Core\Mail\MailManager;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\DelayedRequeueException;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Send queued daily digest emails.
 *
 * @QueueWorker(
 * id = "daily_digest_email_queue",
 * title = @Translation("Forum subscription daily digest email queue
 *   processor"), cron = {"time" = 100}
 * )
 */
class DailyDigestEmailQueue extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManager
   */
  private $mail;

  /**
   * Queue worker constructor.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Mail\MailManager $mail
   *   The mail manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MailManager $mail) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->mail = $mail;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.mail')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    if ($this->userMidnight($data)) {
      $body = $data['header'];
      if (isset($data['message_forum']) && is_array($data['message_forum'])) {
        $body .= $this->buildNotificationTypeBlock($data['message_forum'], 'New post notifications');
      }
      if (isset($data['message_topic']) && is_array($data['message_topic'])) {
        $body .= $this->buildNotificationTypeBlock($data['message_topic'], 'New comment notifications');
      }

      $params['to'] = $data['to'];
      $params['title'] = $data['title'];
      $params['message'] = $body;
      $this->mail->mail('forum_notifications_subscription', 'daily_digest_email_queue', $data['to'], $data['langcode'], $params, NULL, TRUE);
    }
    else {
      throw new DelayedRequeueException();
    }
  }

  /**
   * Check if its middle night of the user time zone.
   *
   * @param array $data
   *   The array containing the messages.
   *
   * @return bool
   *   TRUE if the user is in the middle of the night, FALSE otherwise.
   *
   * @throws \Exception
   */
  private function userMidnight(array $data): bool {
    if (!isset($data['user_id']) || !isset($data['created'])) {
      return TRUE;
    }

    $time_zone = User::load($data['user_id'])->getTimeZone();
    $date = new \DateTime("now", new \DateTimeZone($time_zone));
    $created = DateTimePlus::createFromTimestamp($data['created']);

    return $date->format('Y-m-d') != $created->format('Y-m-d');
  }

  /**
   * Build the notification type block.
   *
   * @param array $data
   *   The array containing the messages.
   * @param string $title
   *   The title of the notification type.
   *
   * @return string
   *   The block of text to be added to the email.
   */
  private function buildNotificationTypeBlock(array $data, string $title): string {
    $block = '<br/><br/><br/>----------------------------------<br/>' . t('@title', ['@title' => $title]);
    foreach ($data as $msg) {
      $block .= '<br/>&nbsp;&nbsp;• ' . $msg;
    }

    return $block;
  }

}
